--IMPORTANT-READ THESE TERMS CAREFULLY BEFORE UTILIZING THE INFORMATION CONTAINED IN THIS APPLICATION NOTE. 

--NO SUPPORT/NO WARRANTY.  
--THE DESIGN FILES AND DOCUMENTATION ARE PROVIDED WITHOUT SUPPORT OR WARRANTY OF ANY KIND.  
--ACTEL IS NOT OBLIGATED TO PROVIDE UPDATES, BUG FIXES, OR TECHNICAL SUPPORT AND DISCLAIMS ALL WARRANTIES INCLUDING, 
--WITHOUT LIMITATION, THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND WARRANTIES OF 
--NON-INFRINGEMENT OF THE RIGHTS OF THIRD PARTIES (INCLUDING, WITHOUT LIMITATION, RIGHTS UNDER PATENT, COPYRIGHT, 
--TRADE SECRET, OR OTHER INTELLECTUAL PROPERTY RIGHTS).  RECIPIENT ACCEPTS THE APPLICATION NOTE DESIGN FILES AND 
--DOCUMENTATION IN "AS-IS" CONDITION.


-- FSM.vhd

library IEEE;
use IEEE.STD_Logic_1164.all;

entity FSM is
    port (PCLK, PRESET_N: in std_logic;
          DUTY_CYC: in std_logic_vector (2 downto 0);
          PADDR: out std_logic_vector (4 downto 0);
          PWDATA: out std_logic_vector (7 downto 0));
end entity FSM;

architecture RTL of FSM is
    --Configuration States of FSM
    type CurrentState is (S_RESET,S_CONFIG_PRE,S_CONFIG_PER,S_CONFIG_EN,S_CONFIG_MASK,S_RUN_POS,S_RUN_NEG);
    signal STATE: CurrentState;
    constant PRESCALE: std_logic_vector(7 downto 0):="00000001"; --PRESCALE=1
    constant PERIOD: std_logic_vector(7 downto 0):="00001110";   --PERIOD=14
    constant PWM_ENABLE: std_logic_vector(7 downto 0):="00000001";
    constant INT_MASK: std_logic_vector(7 downto 0):="00000000";
begin
    FSM_CORE: process (PCLK)
    begin 
        if rising_edge (PCLK) then
            if (PRESET_N='0') then
                STATE <= S_RESET;
            end if;
            case (STATE) is
                when S_RESET => if (PRESET_N='1') then
                                    STATE <= S_CONFIG_PRE;
                                end if;
                --PRESCALE Register Configuration
                when S_CONFIG_PRE =>  PADDR <= "00000";
                                      PWDATA <= PRESCALE;
                                      STATE <= S_CONFIG_PER;
                --PERIOD Register Configuration
                when S_CONFIG_PER =>  PADDR <= "00001";
                                      PWDATA <= PERIOD;
                                      STATE <= S_CONFIG_EN;
                --CorePWM Enable Reg Configuration
                when S_CONFIG_EN =>   PADDR <= "00010";
                                      PWDATA <= PWM_ENABLE;
                                      STATE <= S_CONFIG_MASK;
                --INT Register Configuration
                when S_CONFIG_MASK => PADDR <= "00011";
                                      PWDATA <= INT_MASK;
                                      STATE <= S_RUN_POS; 
                --Positive Edge Register Configuration 
                when S_RUN_POS =>     PADDR <= "00101";
                                      case DUTY_CYC is
                                      --For Register Values based on Duty Cycle Input, refer to Table 2
                                        when "000" => PWDATA(3 downto 0) <= "1111";
                                        when "001" => PWDATA(3 downto 0) <= "0000";
                                        when "010" => PWDATA(3 downto 0) <= "0000";
                                        when "011" => PWDATA(3 downto 0) <= "0000";
                                        when "100" => PWDATA(3 downto 0) <= "0000";
                                        when "101" => PWDATA(3 downto 0) <= "0000";
                                        when others=> PWDATA(3 downto 0) <= "0000";
                                      end case;
                                      STATE <= S_RUN_NEG;
                --Negative Edge Register Configuration 
                when S_RUN_NEG =>     PADDR <= "00110";
                                      case (DUTY_CYC) is
                                      --For Register Values based on Duty Cycle Input, refer to Table 2
                                        when "000" => PWDATA(3 downto 0) <= "0000";
                                        when "001" => PWDATA(3 downto 0) <= "0011";
                                        when "010" => PWDATA(3 downto 0) <= "0110";
                                        when "011" => PWDATA(3 downto 0) <= "1001";
                                        when "100" => PWDATA(3 downto 0) <= "1100";
                                        when "101" => PWDATA(3 downto 0) <= "1111";
                                        when others=> PWDATA(3 downto 0) <= "0000";
                                      end case;
                                      STATE <= S_RUN_POS;
            end case;
        end if;
    end process;
end RTL;
  